@@ -258,7 +258,6 @@ impl Graphics {
                     )
                     .enabled_layer_names(&enable_layers)
                     .enabled_extension_names(&enable_exts),
-                None,
             )
         };
 
@@ -336,7 +335,7 @@ impl Graphics {
             RawWindowHandle::Android(handle) => {
                 if !self.instance.enabled().khr_android_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Android,
+                        window: RawWindowHandleKind::of(&window),
                         source: Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_KHR_android_surface",
                         }),
@@ -355,12 +354,12 @@ impl Graphics {
             }
 
             #[cfg(target_os = "ios")]
-            RawWindowHandle::IOS(handle) => {
+            RawWindowHandle::UiKit(handle) => {
                 todo!()
             }
 
             #[cfg(target_os = "macos")]
-            RawWindowHandle::MacOS(handle) => {
+            RawWindowHandle::AppKit(handle) => {
                 use core_graphics_types::{base::CGFloat, geometry::CGRect};
                 use objc::{
                     class, msg_send,
@@ -370,7 +369,7 @@ impl Graphics {
 
                 if !self.instance.enabled().mvk_macos_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Windows,
+                        window: RawWindowHandleKind::of(&window),
                         source: Some(Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_MVK_macos_surface",
                         })),
@@ -436,7 +435,7 @@ impl Graphics {
             RawWindowHandle::Wayland(handle) => {
                 if !self.instance.enabled().khr_wayland_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Windows,
+                        window: RawWindowHandleKind::of(&window),
                         source: Some(Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_KHR_wayland_surface",
                         })),
@@ -465,10 +464,10 @@ impl Graphics {
             }
 
             #[cfg(target_os = "windows")]
-            RawWindowHandle::Windows(handle) => {
+            RawWindowHandle::Win32(handle) => {
                 if !self.instance.enabled().khr_win32_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Windows,
+                        window: RawWindowHandleKind::of(&window),
                         source: Some(Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_KHR_win32_surface",
                         })),
@@ -476,9 +475,11 @@ impl Graphics {
                 }
 
                 unsafe {
-                    let mut info = Win32SurfaceCreateInfoKHR::default();
-                    info.hinstance = handle.hinstance;
-                    info.hwnd = handle.hwnd;
+                    let info = Win32SurfaceCreateInfoKHR {
+                        hinstance: handle.hinstance,
+                        hwnd: handle.hwnd,
+                        ..Default::default()
+                    };
                     self.instance.create_win32_surface_khr(&info, None)
                 }
                 .result()
@@ -489,6 +490,14 @@ impl Graphics {
                 })?
             }
 
+            #[cfg(target_os = "windows")]
+            RawWindowHandle::WinRt(_) => {
+                return Err(CreateSurfaceError::UnsupportedWindow {
+                    window: RawWindowHandleKind::of(&window),
+                    source: Some(Box::from("WinRT is not supported")),
+                })
+            }
+
             #[cfg(any(
                 target_os = "linux",
                 target_os = "dragonfly",
@@ -499,7 +508,7 @@ impl Graphics {
             RawWindowHandle::Xcb(handle) => {
                 if !self.instance.enabled().khr_xcb_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Xcb,
+                        window: RawWindowHandleKind::of(&window),
                         source: Some(Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_KHR_xcb_surface",
                         })),
@@ -537,7 +546,7 @@ impl Graphics {
             RawWindowHandle::Xlib(handle) => {
                 if !self.instance.enabled().khr_xlib_surface {
                     return Err(CreateSurfaceError::UnsupportedWindow {
-                        window: RawWindowHandleKind::Xlib,
+                        window: RawWindowHandleKind::of(&window),
                         source: Some(Box::new(RequiredExtensionIsNotAvailable {
                             extension: "VK_KHR_xlib_surface",
                         })),
@@ -547,7 +556,7 @@ impl Graphics {
                 unsafe {
                     self.instance.create_xlib_surface_khr(
                         &XlibSurfaceCreateInfoKHR {
-                            window: handle.window.into(),
+                            window: handle.window,
                             dpy: handle.display,
                             ..XlibSurfaceCreateInfoKHR::default()
                         },
@@ -568,7 +577,7 @@ impl Graphics {
                 );
 
                 return Err(CreateSurfaceError::UnsupportedWindow {
-                    window: RawWindowHandleKind::Unknown,
+                    window: RawWindowHandleKind::of(&window),
                     source: None,
                 });
             }
