@@ -315,7 +315,7 @@ impl<'a> EncoderCommon<'a> {
     pub fn bind_graphics_descriptors<P>(
         &mut self,
         layout: &'a P,
-        descriptors: &'a impl UpdatedPipelineDescriptors<P>,
+        descriptors: &impl UpdatedPipelineDescriptors<P>,
     ) where
         P: TypedPipelineLayout,
     {
@@ -345,7 +345,7 @@ impl<'a> EncoderCommon<'a> {
     pub fn bind_compute_descriptors<P>(
         &mut self,
         layout: &'a P,
-        descriptors: &'a impl UpdatedPipelineDescriptors<P>,
+        descriptors: &impl UpdatedPipelineDescriptors<P>,
     ) where
         P: TypedPipelineLayout,
     {
@@ -375,7 +375,7 @@ impl<'a> EncoderCommon<'a> {
     pub fn bind_ray_tracing_descriptors<P>(
         &mut self,
         layout: &'a P,
-        descriptors: &'a impl UpdatedPipelineDescriptors<P>,
+        descriptors: &impl UpdatedPipelineDescriptors<P>,
     ) where
         P: TypedPipelineLayout,
     {
@@ -501,6 +501,9 @@ impl<'a> Encoder<'a> {
     where
         T: Pod,
     {
+        assert_eq!(offset % 4, 0);
+        assert!(size_of_val(data) <= 65_536, "Data length greater than 65536 MUST NOT be uploaded with encoder, consider buffer mapping. Actual data is {} bytes", data.len());
+
         if data.is_empty() {
             return;
         }
@@ -519,18 +522,6 @@ impl<'a> Encoder<'a> {
         )
     }
 
-    /// Updates a buffer's contents from host memory
-    pub fn update_buffer_alloc<T>(&mut self, buffer: &'a Buffer, offset: u64, data: &'a [T])
-    where
-        T: Pod,
-    {
-        if data.is_empty() {
-            return;
-        }
-
-        self.update_buffer(buffer, offset, data);
-    }
-
     /// Builds acceleration structures.
     pub fn build_acceleration_structure(
         &mut self,
@@ -589,6 +580,14 @@ impl<'a> Encoder<'a> {
         dst_buffer: &'a Buffer,
         regions: &'a [BufferCopy],
     ) {
+        #[cfg(debug_assertions)]
+        {
+            for region in regions {
+                assert!(src_buffer.info().size >= region.src_offset + region.size);
+                assert!(dst_buffer.info().size >= region.dst_offset + region.size);
+            }
+        }
+
         self.inner.commands.push(
             self.inner.scope,
             Command::CopyBuffer {
@@ -731,7 +730,7 @@ impl<'a> Encoder<'a> {
     /// buffer.
     pub fn finish(mut self) -> CommandBuffer {
         self.command_buffer
-            .write(self.inner.commands.drain())
+            .write(self.inner.commands.drain(), self.inner.scope)
             .expect("TODO: Handle command buffer writing error");
 
         self.command_buffer
@@ -802,7 +801,7 @@ impl<'a, 'b> RenderPassEncoder<'a, 'b> {
                 .set_viewport(self.framebuffer.info().extent.into());
         }
 
-        let gp = pipeline.get(&self.render_pass, self.subpass, device)?;
+        let gp = pipeline.get(self.render_pass, self.subpass, device)?;
         self.inner.bind_graphics_pipeline(gp);
         Ok(())
     }
