@@ -196,7 +196,7 @@ impl Spirv {
 impl From<Spirv> for ShaderModuleInfo {
     fn from(shader: Spirv) -> Self {
         ShaderModuleInfo {
-            code: shader.code.into(),
+            code: shader.code,
             language: ShaderLanguage::SPIRV,
         }
     }
@@ -357,7 +357,7 @@ pub enum CreateShaderModuleError {
     #[error("Failed to validate shader module")]
     NagaValidationError {
         #[from]
-        source: naga::valid::ValidationError,
+        source: naga::WithSpan<naga::valid::ValidationError>,
     },
 }
 
